function make_inpfile_nc(face,param,filename,removedbl)
  %% lets try to make this the most general way to create meshes for
  %% NC.inp

  %% the main idea behind the construction of the mesh, that the mesh
  %% is split into multiple faces that are  each face has its nodes,
  %% its elements and its boundary conditions
  %% face should contain the coords of the nodes on each face and
  %% the elements. however faces may share a common edge, thus nodes
  %% may be double, also for each face the nodenumbering starts with
  %% 1. Correct that with remove_patches
  %% this files creates the input file NC.inp as well as the file
  %% TheNodes.txt containing the information an all nodes and the file
  %% TheElements.txt containing the element information
  %%
  %% To be honest, I was too lazy to split BEM and Evalelements into
  %% different files, which in the long run, would have been helpful
  %% for the output. But the trick with not removing dbl nodes also works
  %%
  %% what a face is supposed to contain
  %%     nodes    coords of the nodes
  %%     elem     definitions of elements starting with node number 1
  %%              if we have triangles the 4th entry in elem = -1
  %%     bytpye   VELO, ADMI, or PRES
  %%     bcval    value of the bc, can be a vector if middle face
  %%              elements are considered
  %%     bccurve  curve definitions for the bc conditions, default
  %%              [-1,-1;-1,-1];
  %%     group    groupnumber
  %%     type     0 = surf, 1 = middle, 2 = eval
  %%
  %% Copyright (C) 2022 W. Kreuzer <wolfgang.kreuzer@oeaw.ac.at>.
  %%
  %%
  %% This program is free software: you can redistribute it and/or modify
  %% it under the terms of the GNU General Public License as published by
  %% the Free Software Foundation, either version 3 of the License, or
  %% (at your option) any later version.
  %
  %% This program is distributed in the hope that it will be useful,
  %% but WITHOUT ANY WARRANTY; without even the implied warranty of
  %% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  %% GNU General Public License for more details.
  %%
  %% You should have received a copy of the GNU General Public License
  %% along with this program.  If not, see <http://www.gnu.org/licenses/>.
  
  if(nargin < 4)
    removedbl = 1;
    if(nargin < 3)
      filename = 'NC.inp';    
      if(nargin < 2)
	param = [];
	if(nargin < 1)
	  error("Give me at least the definition of nodes and elements");
	end
      end
    end
  end

  
  if isempty(filename)
    filename = "NC.inp";
  end  

  % now this is cheating, since we know that the sphere is defined in
  % face{1}. There is no intersection of face{1} with the other faces,
  % that contain evaluation nodes. We do not need to care about double
  % nodes there, so forget about removing double nodes
  % why wouldn't we want to remove double nodes? because in order to
  % do that, the nodelist would be ordered with respect to the
  % x-coordinate and we would loose this easy splitting between sphere
  % and evaluationgrid
  % if you are not satisfied with that, you could create two files:
  % one containing the sphere and the other containing the evaluation mesh
  removedbl = 0;
  [N,E] = remove_patches(face,removedbl);
  
  fh = fopen(filename,'w');
  nfaces = length(face);


  %% Default parameters
  %% give the mesh a name 
  p.name = "Some Mesh";
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%              Units
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  
  p.bunit = 0;    %  m N and stuff
  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%              Freqs
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  %% these are default values, they will be later overwritten by the
  %% parameters defined in param
  p.n_freq = 1;    
  p.freqstep = 0.1;
  p.freq0 = 0.1;
  
				% 
				%if(freqband)
				%    freqcrv = [0.1 100.0 0.0;
				%               0.2 169.99 0.0;
				%               0.3 243.10 0.0;
				%               0.4 339.87 0.0;
				%               0.5 418.14 0.0;
				%               0.6 500.0 0.0];
				%else
  p.freqcrv = [0.0 0.0 0.0;
               1.0 1000.0 0.0];
				%end

  p.freqcrvnumber = 0;
  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   Symmetric planes and numerical parameters, info about mesh
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  
  p.mi_expand = 8;   % min number of terms in the FMM
  p.nsympl = 0;      % number of symmetric planes
  p.nprint = 2;
  p.ifinci = 1;      % external sources are included in the result
  p.ipmethod = 4;    % multipole or not or which one TBEM = 0, FMM = 4
  p.isolver = 0;     % CGS vs direct solver  0 CGS 4 LAPACK
  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  
				% calculate the rest of the data
  for i = 1:nfaces
    g(i) = face{i}.group;
  end
  p.ngroups = max(g)+1;  % lets assume the groups have not holes in
                       % them and start with zero
  p.nelt = size(E,1);
  p.nnode = size(N,1);
  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%     Info about the incoming wave
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  
  p.nincwa = 1;       % number plane waves
  p.npsouc = 0;       % number point sources
  p.ifinte = 0;       % exterior problem 
  p.ClusLen = 0;      % edge length of cluster, of = 0  it will be
                      % computed automatically
  p.iprecond = 0;     % preconditioning 
  p.dir = [0,0,1];         % matrix containing the directions of the different
                      % plane waves
  p.strplan = 1;     % vector containing the complex valued strength of
                      % the plane wave
  p.curplan = [-1 -1];     % matrix containing the number of the frequency
                    % curve for freq, depend. strengh, curplan should
                    % have 2 columns, one for the real and one for the
                    % imag part, -1 means no curve is given
  p.pos = [30.0 0.0 0.0]; % matrix containing the coordinates of the
			  % pointsources
  p.strpoi = 1e-4;     % vector containiing the complex valued strengh
                       % of the point source
  p.curpoi = [-1,-1]; % matrix containing the number of the frequency
                      % curve for freq, depend. strengh, curpoi should
                    % have 2 columns, one for the real and one for the
                    % imag part, -1 means no curve is given 
  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%       Info about the world
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  
  p.sspeed = 340;
  p.densmd = 1.3;
  p.ruimag = 1.0;
  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%         Symmetrystuff
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  
  if(p.nsympl)
    p.isymp = [3 0 3];
    p.xsympl = [0 0 0];
  end
  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   Info about the curves (BC and ext. source)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  
  p.nCurves = 0;           % number of curves, not really necessary
  
  p.curves = [];           % three dimensional array containing the
			   % curve data: 
			   %    first line contains 
			   %    curves(number of curve, rows in this
			   %    curve), the rest of the lines contain
			   %    curves(freq,val,ncurv)
  
				%curves(:,:,1) = [1 5
				%                 1 1
				%                 2 2
				%                 3 3
				%                 4 4
				%                 5 5];
				%curves(:,:,2) = [2 5
				%                 1 1
				%                 2 2
				%                 3 3
				%                 4 4
				%                 5 5];
  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%             Definition part is over 
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


  
  %% lets check if something was set in param
  %% and also change the default p values then
  if(!isempty(param))
    names = fieldnames(param);
    for i = 1:length(names),
      val = getfield(param,names{i});
      % don't care if it exists yet
      p = setfield(p,names{i},val);
    end
  end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  write the headers
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  fprintf(fh, 'Mesh2HRTF 0.5.0\n');
  fprintf(fh, '%s\n', p.name);
  fprintf(fh, '##\n');
  fprintf(fh,'## Controlparameters I\n');
  fprintf(fh, '##\n');
  fprintf(fh,'0 0 0 0 7 %d\n',p.bunit);
  fprintf(fh, '##\n');
  fprintf(fh, '## Controlparameter II \n');
  fprintf(fh, ['## (nAnalysisMode, nTimeSteps, dTimeIncrement, dTimeStart, ' ...
		 'nOutputSteps, nOutputType, nMakeRestart)\n']);
  fprintf(fh,'1 %d %e %d 1 0 0\n', p.n_freq, p.freqstep, p.freq0);
  fprintf(fh, '##\n');
  fprintf(fh,'## Load Frequency Curve \n');
  fprintf(fh,'## (nCurveIndex, nPoiCurv)\n');
 
  fprintf(fh,'%d %d\n', p.freqcrvnumber, size(p.freqcrv,1));
  fprintf(fh,'%e %e %e\n',p.freqcrv');
  fprintf(fh, '##\n');
  fprintf(fh, '## 1. Main Parameters I \n');
  fprintf(fh, ['## (nBGroups, nBElements, nBNodes, nBChiefPoints, ' ...
		 'nLevelsSym, nOutputMode, nBChiefPointMode, nViscotubes, ' ...
		 'nViscolayers)\n']);
  fprintf(fh, '%d %d %d %d %d %d %d %d %d\n', p.ngroups, p.nelt, p.nnode, ...
          p.mi_expand, p.nsympl, p.nprint, p.ifinci, p.ipmethod, p.isolver);
  fprintf(fh, '##\n');
  fprintf(fh, '## 2. Main Parameters II\n');
  fprintf(fh,'%d %d %d 0.0e0 0 0 0\n', p.nincwa, p.npsouc, p.ifinte);
  fprintf(fh, '##\n');
  fprintf(fh, '## 3. Main Parameters III\n');
  fprintf(fh, '0 0 0 0\n');
  fprintf(fh, '##\n');
  fprintf(fh, '## 4. Main Parameters IV\n');
  fprintf(fh, '## dSoundSpeed, dDensity, dRuimag\n');
  fprintf(fh, '%e %e %e 0.0e0 0.0e0 0.0e0\n',p.sspeed, p.densmd, p.ruimag);
  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%          Write the Nodes
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  
  fprintf(fh, 'NODES\n');
  if(p.bunit == 1)
    N = N*1000;  % mm instead of m
  end
  nnodes = rows(N);
  N = [[1:nnodes]',N];
  

  fh2 = fopen("TheNodes.txt","w");
  if(!fh2)
    error("Sorry could not open TheNodes.txt");
  end
  fprintf(fh2,"%d\n",rows(N));
  fprintf(fh2,'%d %e %e %e\n', N');
  fclose(fh2);
  %% tell NC.inp the name of this file, fh is the handle for NC.inp
  fprintf(fh,"TheNodes.txt\n");

  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%          Write the Elements
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

  
  fprintf(fh, 'ELEMENTS\n');
  %%
  %% do it facewise
  %%
  
  E = [ [1:size(E,1)]', E ];
  start = 1;
  [m,n] = size(E);
  
  for i = 1:nfaces
    nelems = size(face{i}.elem,1);
    E(start:start+nelems-1,n + 1) = face{i}.type;
    E(start:start+nelems-1,n + 2) = 0;
    E(start:start+nelems-1,n + 3) = face{i}.group;
    start = start + nelems; %remember face numbers need to be unique,
			    %but each face starts with elem number 1,
			    %thus shift the numbers for the faces > 1
  end

  fh2 = fopen("TheElements.txt","w");
  if(!fh2)
    error("Sorry could not open TheElements.txt");
  end
  fprintf(fh2,"%d\n",rows(E));
  if( n == 4 )   % triangles nodes + elem number + elem type
    fprintf(fh2,'%d %d %d %d %d %d %d\n',E');
  else
    %% there may be a mixture, we will have to check
    %% takes time, but I have currently no better idea
    for i = 1:rows(E),
      if (E(i,5) == -1)
	fprintf(fh2,'%d %d %d %d %d %d %d\n',[E(i,1:4),E(i,6:8)]);
      else
	fprintf(fh2,'%d %d %d %d %d %d %d %d\n',E(i,:));
      end
    end
  end
  fclose(fh2);
  fprintf(fh,"TheElements.txt\n");
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%           Symmetry
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  if(p.nsympl)
    fprintf(fh,'SYMMETRY\n');
    fprintf(fh,'%d %d %d\n', p.isymp);
    fprintf(fh,'%d %d %d\n', p.xsympl);
    fprintf(fh,'\n');
  end
  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%           Boundary
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  
  fprintf(fh,'BOUNDARY\n');
  start = 1;
  for i = 1:nfaces,
    if( face{i}.type == 2) 
      continue;      % eval elements do not need boundary conditions
    end
    
    if( face{i}.bctype == 'VELO' && ( sum( abs(face{i}.bcval) ) < ...
                                      1e-10 ) )
      continue;     % implizit neuman bc is assumed anyway
    end
    nelems = size(face{i}.elem,1);
    fprintf(fh, 'ELEM %d TO %d %s %e ', start, start + nelems-1, face{i}.bctype, real(face{i}.bcval(1)) );

    fprintf(fh, '%d ', face{i}.bccurve(1) );
    fprintf(fh, '%e ',imag(face{i}.bcval(1)) );
    fprintf(fh, '%d ', face{i}.bccurve(2));
    
    fprintf(fh,'\n'); 
    start = start + nelems;
  end
  fprintf(fh, 'RETU\n');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%            External Source
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  
  if(p.nincwa) % there is a plane wave
    fprintf(fh,'PLANE WAVES\n');
    A = [0:p.nincwa-1]';
    A = [A p.dir];
    A = [A real(p.strplan) p.curplan(:,1) imag(p.strplan) p.curplan(:,2)];
    fprintf(fh, '%d %e %e %e %e %d %e %d\n', A');
  end
  %%
  %%
  %%
  if(p.npsouc)
    fprintf(fh,'POINT SOURCE\n');
    A = [0:p.npsouc-1]';
    A = [A,p.pos];
    A = [A,real(p.strpoi),p.curpoi(:,1),imag(p.strpoi),p.curpoi(:,2)];
   %    A = [A,real(strplan),curplan(:,1),imag(strplan),curplan(:,2)];
    fprintf(fh, '%d %e %e %e %e %d %e %d\n', A');
  end
  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                Curves
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  
  if(p.nCurves)
    fprintf(fh,'CURVES\n');
    [n1,n2,n3] = size(p.curves);
    if(n3 ~= nCurves)
      error('Input for CURVES is wrong');
    end
    fprintf(fh, '%d %d\n', p.nCurves, n1);
    for i = 1:n3,
      ncurv = p.curves(1,1,i);
      lcurv = p.curves(1,2,i);
      fprintf(fh, '%d %d\n', p.curves(1,1:2,i));
      fprintf(fh, '%e %e\n', p.curves(2:lcurv+1,:,i)');
    end
  end
  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%           Rest of the best
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  
  fprintf(fh,'POST PROCESS\n');
  fprintf(fh,'##\n');
  fprintf(fh,'## BEM end of input ----\n');
  fprintf(fh,'END\n');
  fprintf(fh,'##\n');
  fprintf(fh,'## this line seems to be important otherwise you end up in an\n');
  fprintf(fh,['## endless loop, or at least add a return after END\' ...
		'n']);
  fclose(fh);
  
end
