function [N,E1]=remove_patches(face,removedbl)
  %% merges all faces,
  %% it is assumed that the mesh is split into different faces, where
  %% each face contains it nodes, elements and boundary conditions
  %% the faces are independent of each other, thus, they may use the
  %% same node and element numbers, thus they have to be merged
  %% accordingly, also faces can contain the same nodes, i.e. if the
  %% intersect, these nodes need to be removed, at least for the BEM
  %% elements and nodes
  %%
  %% concate everything, raises the nodenumbers in the elements and
  %% remove double nodes
  %% also the face number is added in the last column, this helps in
  %% setting the boundary condition later
  %% if you mix faces with quadrilaterals and triangles the dimension
  %% of E1 is 5, for the triangles -1 is written in the 4th column
  %% 
  %%  N dimension number of nodes times 3
  %%  E1 number of elements times dimension (max element vertices) + 1
  %% 
  %%  face numbers will not be added
  %%
  %% Copyright (C) 2022 W. Kreuzer <wolfgang.kreuzer@oeaw.ac.at>.
  %%
  %%
  %% This program is free software: you can redistribute it and/or modify
  %% it under the terms of the GNU General Public License as published by
  %% the Free Software Foundation, either version 3 of the License, or
  %% (at your option) any later version.
  %%
  %% This program is distributed in the hope that it will be useful,
  %% but WITHOUT ANY WARRANTY; without even the implied warranty of
  %% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  %% GNU General Public License for more details.
  %%
  %% You should have received a copy of the GNU General Public License
  %% along with this program.  If not, see <http://www.gnu.org/licenses/>.
  if(nargin < 2)
    removedbl = 1;
  end
  
  nfaces = length(face);
  N = [];
  for i = 1:nfaces
      N1 = face{i}.nodes;
      N = [N;N1'];
  end    


  if(removedbl) 
    N=round(N*1e6)*1e-6;
    % in principle relatively fast, but it orders the nodelist
    [N1,i1,i2]=unique(N,'rows');
    N=N1;
  end

  
  E = []; elemface = []; addnumber = 0;
 
  % raise the node numbers with the patches
  for i=1:nfaces
  %  keyboard
    E1 = face{i}.elem(:,:);
  
    %% set the facenumber for each element
    %% we need that later to asign the right bc
    eface(1:size(E1,1),1) = i;   % rows for octave , size for matlab
			      %    E1(counter:rows(E1)+counter,5) = i;
			      %      n=rows(E);
    %% we have now a slight problem, what happens if different faces
    %% have different number of elements
    if( columns(E)  < columns(E1) )
      %% the new face has quadilaterals, the old ones only triangles
      %% insert -1 as 4th node
      if( i > 1)
	E(:,end + 1) = -1;
      end
    end
    

    if( columns(E1) < columns(E) )
      %% E contains quadrilaterals, E1 triangles
      E1(:,end+1) = -1;
      E1(:,1:3) = E1(:,1:3) + addnumber;
    else
      E1 = E1 + addnumber;
    end
    E = [E;E1];
    
    elemface = [elemface;eface];
    addnumber = addnumber + size(face{i}.nodes,2);
    clear eface;
  end
  
  %% replace the double node numbers
  if(removedbl)
    E1=-1 * ones(size(E));
    for j=1:length(i2)
      E1(E==j) = i2(j);
    end
    %E1(:,end+1) = elemface;
  else
    E1 = E;
  end
  
end
